#!/usr/bin/env python3
import cloudpickle
from sys import stderr
from os import getenv
from dotenv import load_dotenv

from twisted.internet.protocol import Protocol, Factory
from twisted.internet import reactor
from twisted.internet.endpoints import TCP4ServerEndpoint


class CortexServer(Protocol):
    def __init__(self, factory):
        self.factory = factory
        print("Starting factory:", self.factory)
        self.HOST = getenv(
            "HOST"
        )  # The server's hostname or IP address - standard loopback interface address (localhost)
        self.PORT = int(
            getenv("PORT")
        )  # Port to listen on (non-privileged ports are > 1023)
        self.USERNAME = getenv("USERNAME")
        self.SECRET_KEY = getenv("SECRET_KEY")
        self.CPU_COUNT = 0
        self.IP = ""
        self.DATETIME_CONNECTED = ""
        self.AUTHENTICATED = False
        self.CLIENT_ID = self.factory.totalProtocols

    def connectionMade(self):
        print("Starting connection #:", self.factory.numProtocols)
        self.factory.numProtocols = self.factory.numProtocols + 1
        self.factory.totalProtocols = self.factory.totalProtocols + 1
        print("Number of connections:", self.factory.numProtocols)
        self.factory.clients.append(self)
        packet = cloudpickle.dumps(
            {
                "GREETING": f"Welcome! There are currently {self.factory.numProtocols} open connections.\n",
                "CLIENT_ID": self.CLIENT_ID,
            }
        )
        self.transport.write(packet)

    def connectionLost(self, reason):
        self.factory.numProtocols = self.factory.numProtocols - 1
        print(f"Connection lost: {reason}")
        print(f"{self.factory.numProtocols} clients connected.")
        self.factory.clients.remove(self)

    def dataReceived(self, data):
        data = cloudpickle.loads(data)
        print("RECEIVED:", data)
        if "USERNAME" in data and "SECRET_KEY" in data:
            if (
                data["USERNAME"] == self.USERNAME
                and data["SECRET_KEY"] == self.SECRET_KEY
            ):
                # The user is authenticated to distribute commands.
                self.AUTHENTICATED = True
                self.IP = data["IP"]
                self.CPU_COUNT = data["CPU_COUNT"]
                self.DATETIME_CONNECTED = data["DATETIME_CONNECTED"]
                self.transport.write(
                    cloudpickle.dumps({"AUTHENTICATED": self.AUTHENTICATED})
                )
                print(f"User {data['USERNAME']} is authenticated.")
                self.factory.cortex = self

                # for client in self.factory.clients:
                # print(client.__dict__)
            else:
                self.transport.write(
                    cloudpickle.dumps({"AUTHENTICATED": self.AUTHENTICATED})
                )
                stderr.write(
                    "This USERNAME and SECRET_KEY cannot be authenticated. Closing connection."
                )
                self.transport.loseConnection()
                # for client in self.factory.clients:
                # print(client.__dict__)
        elif "IP" in data and "CPU_COUNT" in data and self.AUTHENTICATED is False:
            print(data)
            print("Clients:", self.factory.clients)
            self.IP = data["IP"]
            self.CPU_COUNT = data["CPU_COUNT"]
            self.DATETIME_CONNECTED = data["DATETIME_CONNECTED"]
            self.CLIENT_ID = data["CLIENT_ID"]
            # for client in self.factory.clients:
            # print(client.__dict__)
        elif self.AUTHENTICATED is True and "FUNC" in data:
            func = data["FUNC"]
            args = data["ARGS"]
            dep_funcs = data["DEP_FUNCS"]
            modules = data["MODULES"]
            callback = data["CALLBACK"]
            callback_args = data["CALLBACK_ARGS"]
            group = data["GROUP"]
            response_mode = data["RESPONSE_MODE"]
            self.factory.response_mode = response_mode

            self.startJob(func, args)
        elif self.AUTHENTICATED is False and "READY" in data:
            print(f"CLIENT STATUS: {data}")

        elif self.AUTHENTICATED is True and "VERIFY" in data:
            for worker in self.factory.workers:
                try:
                    packet = cloudpickle.dumps(
                        {
                            "ARG": next(self.factory.args),
                            "ARGS_COUNTER": self.factory.args_counter,
                        }
                    )
                    worker.transport.write(packet)
                    print(
                        f"Packet with arg {self.factory.args_counter} sent to {worker.CLIENT_ID}"
                    )
                    self.factory.args_counter = self.factory.args_counter + 1

                except StopIteration:
                    print("The arguments iterable was empty.")

        elif "RESULT" in data:
            print("RESULTS PACKET:", data)
            if self.factory.response_mode == "OBJECT":
                self.factory.results.append(data)
                try:
                    packet = cloudpickle.dumps(
                        {
                            "ARG": next(self.factory.args),
                            "ARGS_COUNTER": self.factory.args_counter,
                        }
                    )
                    self.transport.write(packet)
                    print(
                        f"Packet with arg {self.factory.args_counter} sent to {self.CLIENT_ID}"
                    )
                    self.factory.args_counter = self.factory.args_counter + 1
                except StopIteration:
                    print("The arguments have been exhausted.")
                    if len(self.factory.workers) > 1:
                        if self.factory.lastCounter == 1:
                            self.factory.cortex.transport.write(
                                cloudpickle.dumps(
                                    {
                                        "FINAL_RESULT": self.factory.gatherResults(
                                            self.factory
                                        )
                                    }
                                )
                            )

                        else:
                            self.factory.lastCounter = self.factory.lastCounter - 1
                    elif len(self.factory.workers) == 1:
                        self.factory.cortex.transport.write(
                            cloudpickle.dumps(
                                {
                                    "FINAL_RESULT": self.factory.gatherResults(
                                        self.factory
                                    )
                                }
                            )
                        )
            elif (
                self.factory.response_mode == "STREAM"
                or self.factory.response_mode == "SQLITE"
            ):
                self.factory.cortex.transport.write(cloudpickle.dumps(data))
                try:
                    packet = cloudpickle.dumps(
                        {
                            "ARG": next(self.factory.args),
                            "ARGS_COUNTER": self.factory.args_counter,
                        }
                    )
                    self.transport.write(packet)
                    print(
                        f"Packet with arg {self.factory.args_counter} sent to {self.CLIENT_ID}"
                    )
                    self.factory.args_counter = self.factory.args_counter + 1
                except StopIteration:
                    print("The arguments have been exhausted.")
                    if len(self.factory.workers) > 1:
                        if self.factory.lastCounter == 1:
                            print(
                                "Final results packet has been transmitted to the cortex."
                            )

                        else:
                            self.factory.lastCounter = self.factory.lastCounter - 1
                    elif len(self.factory.workers) == 1:
                        print(
                            "Final results packet has been transmitted to the cortex."
                        )

        elif "GET_CLUSTER_STATUS" in data:
            packet = {"CLUSTER_STATUS": True}
            for client in self.factory.clients:
                packet[str(client.CLIENT_ID)] = {}
                packet[str(client.CLIENT_ID)]["CLIENT_ID"] = str(client.CLIENT_ID)
                packet[str(client.CLIENT_ID)]["HOST"] = str(client.HOST)
                packet[str(client.CLIENT_ID)]["PORT"] = str(client.PORT)
                packet[str(client.CLIENT_ID)]["CPU_COUNT"] = str(client.CPU_COUNT)
                packet[str(client.CLIENT_ID)]["IP"] = str(client.IP)
                packet[str(client.CLIENT_ID)]["DATETIME_CONNECTED"] = str(
                    client.DATETIME_CONNECTED
                )
                packet[str(client.CLIENT_ID)]["AUTHENTICATED"] = str(
                    client.AUTHENTICATED
                )

            self.factory.cortex.transport.write(cloudpickle.dumps(packet))

        elif "KILL_CLUSTER" in data:
            for client in self.factory.clients:
                client.transport.write(cloudpickle.dumps({"KILL_NODE": "KILL_NODE"}))

            for client in self.factory.clients:
                client.transport.loseConnection()

        elif "KILLED_CLUSTER" in data:
            reactor.stop()

        else:
            print(data)

    def startJob(self, func, args=()):
        # Here is where the magic happens. Hungry consumers - feed them once and they keep
        # asking for more until the args are exhausted.

        # Flush settings in case another job has already been completed in this lifecycle.
        self.factory.args_counter = 0
        self.factory.results = []

        print("MAP FUNC:", func)
        print("MAP ARGS:", args)
        ip_list = []
        ip_map = []
        workers_list = []
        cpu_total = 0
        self.factory.args = iter(args)
        for client in self.factory.clients:
            if client.IP not in ip_list:
                ip_list.append(client.IP)
                ip_map.append((client.IP, client.CPU_COUNT))
                # Int may be unnecessary - check data type.
                cpu_total += int(client.CPU_COUNT)
            if client.AUTHENTICATED is False:
                workers_list.append(client)
        print("IP LIST:", ip_list)
        print("CPU TOTAL", cpu_total)
        print("CLIENTS CONNECTED:", self.factory.numProtocols)
        print(workers_list)
        self.factory.ipMap = ip_map
        self.factory.workers = workers_list
        self.factory.lastCounter = len(self.factory.workers) - 1
        for client in self.factory.workers:
            client.transport.write(cloudpickle.dumps({"START_JOB": True, "FUNC": func}))
        self.factory.cortex.transport.write(cloudpickle.dumps({"PROCEED": True}))


class CortexServerFactory(Factory):

    protocol = CortexServer
    numProtocols = 0
    totalProtocols = 0
    clients = []
    workers = []
    cortex = None
    args = None
    args_counter = 0
    results = []
    lastCounter = 0
    ipMap = []
    response_mode = None

    def buildProtocol(self, addr):
        return CortexServer(factory=CortexServerFactory)

    def gatherResults(self):
        final_results = []
        self.results = sorted(self.results, key=lambda k: k["ARGS_COUNTER"])
        for result in self.results:
            final_results.append(result["RESULT"])
        return final_results


def runCortexServer():
    load_dotenv()
    port = int(getenv("PORT"))
    endpoint = TCP4ServerEndpoint(reactor, port)
    endpoint.listen(CortexServerFactory())
    print(f"ALERT: cortex_server initiated on HOST {getenv('HOST')} at PORT {port}")
    reactor.run()


if __name__ == "__main__":
    runCortexServer()



#!/usr/bin/env python3

import socket
import cloudpickle
from os import getenv
from os.path import dirname, abspath, join

from multiprocessing import Pool
from dotenv import load_dotenv

from twisted.protocols.basic import LineReceiver
from twisted.internet.endpoints import TCP4ClientEndpoint, connectProtocol
from twisted.internet import reactor
import multiprocessing
from datetime import datetime


class AchillesNode(LineReceiver):
    MAX_LENGTH = 999999

    def __init__(self, host, port):

        self.HOST = host  # The server's hostname or IP address
        self.PORT = port  # The port used by the server
        self.connected = False
        self.client_id = -1
        self.func = None
        self.args = []

    def lineReceived(self, data):
        self.handleData(data)

    def handleData(self, data):
        data = cloudpickle.loads(data)
        if "GREETING" in data:
            greeting = data["GREETING"]
            client_id = data["CLIENT_ID"]
            self.client_id = client_id
            print("GREETING:", greeting)
            print("CLIENT_ID", client_id)
            packet = cloudpickle.dumps(
                {
                    "IP": socket.gethostbyname(socket.gethostname()),
                    "CPU_COUNT": multiprocessing.cpu_count(),
                    "DATETIME_CONNECTED": datetime.now(),
                    "CLIENT_ID": self.client_id,
                }
            )
            self.sendLine(packet)
        elif "START_JOB" in data:
            func = data["FUNC"]
            self.func = func
            packet = cloudpickle.dumps({"CLIENT_ID": self.client_id, "READY": True})
            print("START_JOB RESPONSE:", packet)
            self.sendLine(packet)
        elif "ARG" in data:
            print("ARG:", data["ARG"])
            with Pool(multiprocessing.cpu_count()) as p:
                result = p.map(self.func, data["ARG"])
                p.close()
            packet = cloudpickle.dumps(
                {"ARGS_COUNTER": data["ARGS_COUNTER"], "RESULT": result}
            )
            print("RESPONSE PACKET:", packet)
            self.sendLine(packet)
        elif "KILL_NODE" in data:
            self.sendLine(cloudpickle.dumps({"KILLED_CLUSTER": "KILLED_CLUSTER"}))
            self.transport.loseConnection()
            reactor.stop()


def runAchillesNode():
    try:
        if __name__ != "__main__":
            import achilles

            dotenv_path = dirname(achilles.__file__) + "\\lineReceiver\\.env"
        else:
            basedir = abspath(dirname(__file__))
            dotenv_path = join(basedir, ".env")
        load_dotenv(dotenv_path, override=True)
        port = int(getenv("PORT"))
        host = getenv("HOST")

    except BaseException as e:
        print(
            f"No .env configuration file found ({e}). Follow the prompts below to generate one:"
        )
        host, port = genConfig()

    endpoint = TCP4ClientEndpoint(reactor, host, port)
    d = connectProtocol(endpoint, AchillesNode(host, port))

    reactor.run()


def genConfig():
    if __name__ != "__main__":
        import achilles

        dotenv_path = dirname(achilles.__file__) + "\\lineReceiver\\"
    else:
        basedir = abspath(dirname(__file__))
        dotenv_path = join(basedir, ".env")
    host = input("Enter HOST IP address:\t")
    port = int(input("Enter HOST port to connect to:\t"))
    with open(dotenv_path + ".env", "w") as config_file:
        config_file.writelines(f"HOST={host}\n")
        config_file.writelines(f"PORT={port}\n")
        config_file.close()
        print(
            f"Successfully generated .env configuration file at {dotenv_path}.env. Use achilles_node.genConfig() to overwrite."
        )
    return host, port


if __name__ == "__main__":
    runAchillesNode()



#!/usr/bin/env python3

from requests import get
from html2text import html2text


def cortex_function(arg):
    return arg ** 2


def cortex_function2(arg):
    r = get(
        arg,
        headers={"User-Agent": "Mozilla/5.0", "referrer": "https://l.facebook.com/"},
    )
    raw_html = r.text
    markdown_text = html2text(raw_html)

    return markdown_text



#!/usr/bin/env python3

import socket
import cloudpickle
from sys import stderr
import sqlite3
import json

from dotenv import load_dotenv
import yaml
from os import getenv

from twisted.internet.protocol import Protocol
from twisted.internet.endpoints import TCP4ClientEndpoint, connectProtocol
from twisted.internet import reactor
import multiprocessing
from datetime import datetime


class Cortex(Protocol):
    def __init__(self):
        load_dotenv()

        self.HOST = getenv("HOST")  # The server's hostname or IP address
        self.PORT = int(getenv("PORT"))  # The port used by the server
        self.USERNAME = getenv("USERNAME")
        self.SECRET_KEY = getenv("SECRET_KEY")
        self.response_mode = None
        self.sqlite_db_created = False
        self.sqlite_db = ""
        self.args_count = 0
        self.abs_counter = 0

    def dataReceived(self, data):
        data = cloudpickle.loads(data)
        if "GREETING" in data:
            greeting = data["GREETING"]
            print("GREETING:", greeting)
            packet = cloudpickle.dumps(
                {
                    "IP": socket.gethostbyname(socket.gethostname()),
                    "CPU_COUNT": multiprocessing.cpu_count(),
                    "DATETIME_CONNECTED": datetime.now(),
                    "USERNAME": self.USERNAME,
                    "SECRET_KEY": self.SECRET_KEY,
                }
            )
            self.transport.write(packet)

        elif "AUTHENTICATED" in data:
            if data["AUTHENTICATED"] is True:
                print("ALERT: Authentication successful!\n")
                self.command_interface()
            else:
                stderr.write("WARNING: Authentication failed.")
                self.transport.loseConnection()

        elif "PROCEED" in data:
            proceed = input("Press ENTER if ready to proceed:\t")
            if proceed == "":
                print(
                    "PROCEEDING WITH DISTRIBUTING ARGUMENTS AMONGST THE CONNECTED NODES..."
                )
                self.transport.write(cloudpickle.dumps({"VERIFY": True}))
            else:
                stderr.write("ALERT: Job cancelled.")
                self.command_interface()

        elif "RESULT" in data and self.response_mode == "STREAM":
            # If STREAM is your chosen response mode, handle results packets here.
            print(data)
            # Prematurely calls command_interface in certain instances when the last results packet is returned while jobs remain outstanding.
            # if data["ARGS_COUNTER"] == self.args_count - 1:
            # self.command_interface()

        elif "RESULT" in data and self.response_mode == "SQLITE":
            if self.sqlite_db_created is False:
                self.sqlite_db = datetime.now().strftime("%Y%m%d %H%M%S") + ".db"
                conn = sqlite3.connect(self.sqlite_db)
                c = conn.cursor()
                c.execute(
                    "CREATE TABLE results (args_counter real, abs_counter real, result text)"
                )
                self.sqlite_db_created = True
                if len(data["RESULT"]) == 1:
                    try:
                        instruction = f"""INSERT INTO results VALUES ({data['ARGS_COUNTER']}, {self.abs_counter}, {data['RESULT'][0]})"""
                        # print(instruction)
                        c.execute(instruction)
                        self.abs_counter = self.abs_counter + 1
                    except BaseException:
                        instruction = f"""INSERT INTO results VALUES ({data['ARGS_COUNTER']}, {self.abs_counter}, {json.dumps(data['RESULT'][0])})"""
                        # print(instruction)
                        c.execute(instruction)
                        self.abs_counter = self.abs_counter + 1
                    # finally:
                    # if data["ARGS_COUNTER"] == self.args_count - 1:
                    # self.command_interface()
                else:
                    for i in range(len(data["RESULT"])):
                        try:
                            instruction = f"""INSERT INTO results VALUES ({data['ARGS_COUNTER']}, {self.abs_counter}, {data['RESULT'][i]})"""
                            # print(instruction)
                            c.execute(instruction)
                            self.abs_counter = self.abs_counter + 1
                        except BaseException:
                            instruction = f"""INSERT INTO results VALUES ({data['ARGS_COUNTER']}, {self.abs_counter}, {json.dumps(data['RESULT'][i])})"""
                            # print(instruction)
                            c.execute(instruction)
                            self.abs_counter + self.abs_counter + 1

                    # if data["ARGS_COUNTER"] == self.args_count - 1:
                    # self.command_interface()
                c.close()

            else:
                conn = sqlite3.connect(self.sqlite_db)
                c = conn.cursor()
                if len(data["RESULT"]) == 1:
                    try:
                        instruction = f"""INSERT INTO results VALUES ({data['ARGS_COUNTER']}, {self.abs_counter}, {data['RESULT'][0]})"""
                        # print(instruction)
                        c.execute(instruction)
                        self.abs_counter = self.abs_counter + 1
                    except BaseException:
                        instruction = f"""INSERT INTO results VALUES ({data['ARGS_COUNTER']}, {self.abs_counter}, {json.dumps(data['RESULT'][0])})"""
                        # print(instruction)
                        c.execute(instruction)
                        self.abs_counter = self.abs_counter + 1
                    # finally:
                    # if data["ARGS_COUNTER"] == self.args_count - 1:
                    # self.command_interface()
                else:
                    for i in range(len(data["RESULT"])):
                        try:
                            instruction = f"""INSERT INTO results VALUES ({data['ARGS_COUNTER']}, {self.abs_counter}, {data['RESULT'][i]})"""
                            # print(instruction)
                            c.execute(instruction)
                            self.abs_counter = self.abs_counter + 1
                        except BaseException:
                            instruction = f"""INSERT INTO results VALUES ({data['ARGS_COUNTER']}, {self.abs_counter}, {json.dumps(data['RESULT'][i])})"""
                            # print(instruction)
                            c.execute(instruction)
                            self.abs_counter + self.abs_counter + 1
                    # if data["ARGS_COUNTER"] == self.args_count - 1:
                    # self.command_interface()
                c.close()

        elif "FINAL_RESULT" in data:
            print("FINAL RESULT:", data["FINAL_RESULT"])
            self.command_interface()

        elif "CLUSTER_STATUS" in data:
            print("CLUSTER STATUS:", data)
            self.command_interface()

        elif "KILL_NODE" in data:
            stderr.write(
                "ALERT: All cortex_nodes have been disconnected from the cluster. The cortex_server is running and accepting connections."
            )
            reactor.stop()

        else:
            print(data)
            self.command_interface()

    def cortex_compute(
        self,
        cortex_config_path="",
        args=(),
        dep_funcs=(),
        modules=(),
        callback=None,
        callback_args=(),
        group="default",
        response_mode="",
    ):
        cortex_config_path = cortex_config_path
        with open(cortex_config_path, "r") as f:
            cortex_config = yaml.load(f, Loader=yaml.Loader)

        args = cortex_config["ARGS"]
        self.args_count = len(args)
        self.response_mode = response_mode
        dep_funcs = cortex_config["DEP_FUNCS"]
        modules = cortex_config["MODULES"]
        callback = cortex_config["CALLBACK"]
        callback_args = cortex_config["CALLBACK_ARGS"]
        group = cortex_config["GROUP"]
        from cortex_function import cortex_function

        packet = cloudpickle.dumps(
            {
                "FUNC": cortex_function,
                "ARGS": args,
                "DEP_FUNCS": dep_funcs,
                "MODULES": modules,
                "CALLBACK": callback,
                "CALLBACK_ARGS": callback_args,
                "GROUP": group,
                "RESPONSE_MODE": response_mode,
            }
        )
        self.transport.write(packet)

    def get_cluster_status(self):
        packet = cloudpickle.dumps({"GET_CLUSTER_STATUS": "GET_CLUSTER_STATUS"})
        self.transport.write(packet)
        print("ALERT: Requested cluster status.")

    def kill_cluster(self):
        confirm_kill_cluster = input(
            "WARNING: Are you absolutely sure that you want to kill the cluster? Enter YES to proceed."
        )
        if confirm_kill_cluster == "YES":
            packet = cloudpickle.dumps({"KILL_CLUSTER": "KILL_CLUSTER"})
            self.transport.write(packet)
        else:
            stderr.write("ALERT: kill_cluster aborted.")
            self.command_interface()

    def command_interface(self):
        command = input("Cortex cluster is ready to accept commands:\t")
        if command == "cortex_compute":
            cortex_config_path = input(
                "Enter path to cortex_config.yaml to begin job:\t"
            )
            response_mode = input(
                f"Enter desired response mode (OBJECT, SQLITE, or STREAM):\t"
            )
            self.cortex_compute(
                cortex_config_path=cortex_config_path, response_mode=response_mode
            )
        elif command == "cluster_status":
            self.get_cluster_status()
        elif command == "kill_cluster":
            self.kill_cluster()
        elif command == "help":
            print("\n-------\nCommands:")
            print("cortex_compute, cluster_status, kill_cluster, help\n-------\n")
            self.command_interface()
        else:
            print(
                "Sorry, that command is not recognized. Type 'help' for a list of commands."
            )
            self.command_interface()


def runCortex():
    load_dotenv()
    endpoint = TCP4ClientEndpoint(reactor, getenv("HOST"), int(getenv("PORT")))
    d = connectProtocol(endpoint, Cortex())

    reactor.run()


if __name__ == "__main__":
    runCortex()



#!/usr/bin/env python3
import cloudpickle
from dotenv import load_dotenv

import getpass
from sys import stderr
from os import getenv
from os.path import dirname, abspath, join
from types import GeneratorType

from twisted.internet.protocol import Factory
from twisted.protocols.basic import LineReceiver
from twisted.internet import reactor
from twisted.internet.endpoints import TCP4ServerEndpoint


class AchillesServer(LineReceiver):
    MAX_LENGTH = 999999

    def __init__(self, factory):
        self.factory = factory
        print("Starting factory:", self.factory)
        self.HOST = (
            self.factory.HOST
        )  # The server's hostname or IP address - standard loopback interface address (localhost)
        self.PORT = (
            self.factory.PORT
        )  # Port to listen on (non-privileged ports are > 1023)
        self.CPU_COUNT = 0
        self.IP = ""
        self.DATETIME_CONNECTED = ""
        self.AUTHENTICATED = False
        self.CLIENT_ID = self.factory.totalProtocols

    def connectionMade(self):
        print("Starting connection #:", self.factory.numProtocols)
        self.factory.numProtocols = self.factory.numProtocols + 1
        self.factory.totalProtocols = self.factory.totalProtocols + 1
        print("Number of connections:", self.factory.numProtocols)
        self.factory.clients.append(self)
        packet = cloudpickle.dumps(
            {
                "GREETING": f"Welcome! There are currently {self.factory.numProtocols} open connections.\n",
                "CLIENT_ID": self.CLIENT_ID,
            }
        )
        self.sendLine(packet)

    def connectionLost(self, reason):
        self.factory.numProtocols = self.factory.numProtocols - 1
        print(f"Connection lost: {reason}")
        print(f"{self.factory.numProtocols} clients connected.")
        self.factory.clients.remove(self)

    def lineReceived(self, data):
        data = cloudpickle.loads(data)
        print("RECEIVED:", data)
        if "USERNAME" in data and "SECRET_KEY" in data:
            if (
                data["USERNAME"] == self.factory.USERNAME
                and data["SECRET_KEY"] == self.factory.SECRET_KEY
            ):
                # The user is authenticated to distribute commands.
                self.AUTHENTICATED = True
                self.IP = data["IP"]
                self.CPU_COUNT = data["CPU_COUNT"]
                self.DATETIME_CONNECTED = data["DATETIME_CONNECTED"]
                self.sendLine(cloudpickle.dumps({"AUTHENTICATED": self.AUTHENTICATED}))
                print(f"User {data['USERNAME']} is authenticated.")
                self.factory.achilles_controller = self

                # for client in self.factory.clients:
                # print(client.__dict__)
            else:
                self.sendLine(cloudpickle.dumps({"AUTHENTICATED": self.AUTHENTICATED}))
                stderr.write(
                    "This USERNAME and SECRET_KEY cannot be authenticated. Closing connection."
                )
                self.transport.loseConnection()
                # for client in self.factory.clients:
                # print(client.__dict__)
        elif "IP" in data and "CPU_COUNT" in data and self.AUTHENTICATED is False:
            print("Clients:", self.factory.clients)
            self.IP = data["IP"]
            self.CPU_COUNT = data["CPU_COUNT"]
            self.DATETIME_CONNECTED = data["DATETIME_CONNECTED"]
            self.CLIENT_ID = data["CLIENT_ID"]
            # for client in self.factory.clients:
            # print(client.__dict__)
        elif self.AUTHENTICATED is True and "FUNC" in data:
            func = data["FUNC"]
            args = data["ARGS"]
            args_path = data["ARGS_PATH"]
            args_count = data["ARGS_COUNT"]
            modules = data["MODULES"]
            callback = data["CALLBACK"]
            callback_args = data["CALLBACK_ARGS"]
            group = data["GROUP"]
            response_mode = data["RESPONSE_MODE"]
            self.factory.response_mode = response_mode

            self.startJob(func, args, args_path, args_count)
        elif self.AUTHENTICATED is False and "READY" in data:
            print(f"CLIENT STATUS: {data}")

        elif self.AUTHENTICATED is True and "VERIFY" in data:
            self.proceedWithJob()

        elif "RESULT" in data:
            print("RESULTS PACKET:", data)
            if self.factory.response_mode == "OBJECT":
                self.factory.results.append(data)
                try:
                    if isinstance(self.factory.args, GeneratorType):
                        args_counter, arg = next(self.factory.args)
                        packet = cloudpickle.dumps(
                            {"ARG": arg, "ARGS_COUNTER": args_counter}
                        )
                        self.sendLine(packet)
                        print(
                            f"Packet with arg {args_counter} sent to {self.CLIENT_ID}"
                        )
                    else:
                        packet = cloudpickle.dumps(
                            {
                                "ARG": next(self.factory.args),
                                "ARGS_COUNTER": self.factory.args_counter,
                            }
                        )
                        self.sendLine(packet)
                        print(
                            f"Packet with arg {self.factory.args_counter} sent to {self.CLIENT_ID}"
                        )
                        self.factory.args_counter = self.factory.args_counter + 1

                except StopIteration:
                    print("The arguments have been exhausted.")
                    if len(self.factory.workers) > 1:
                        if self.factory.lastCounter == 0:
                            self.factory.achilles_controller.sendLine(
                                cloudpickle.dumps(
                                    {
                                        "FINAL_RESULT": self.factory.gatherResults(
                                            self.factory
                                        )
                                    }
                                )
                            )

                        else:
                            self.factory.lastCounter = self.factory.lastCounter - 1
                    elif len(self.factory.workers) == 1:
                        self.factory.achilles_controller.sendLine(
                            cloudpickle.dumps(
                                {
                                    "FINAL_RESULT": self.factory.gatherResults(
                                        self.factory
                                    )
                                }
                            )
                        )
            elif (
                self.factory.response_mode == "STREAM"
                or self.factory.response_mode == "SQLITE"
            ):
                self.factory.achilles_controller.sendLine(cloudpickle.dumps(data))
                try:
                    if isinstance(self.factory.args, GeneratorType):
                        args_counter, arg = next(self.factory.args)
                        packet = cloudpickle.dumps(
                            {"ARG": arg, "ARGS_COUNTER": args_counter}
                        )
                        self.sendLine(packet)
                        print(
                            f"Packet with arg {args_counter} sent to {self.CLIENT_ID}"
                        )
                    else:
                        packet = cloudpickle.dumps(
                            {
                                "ARG": next(self.factory.args),
                                "ARGS_COUNTER": self.factory.args_counter,
                            }
                        )
                        self.sendLine(packet)
                        print(
                            f"Packet with arg {self.factory.args_counter} sent to {self.CLIENT_ID}"
                        )
                        self.factory.args_counter = self.factory.args_counter + 1

                except StopIteration:
                    print("The arguments have been exhausted.")
                    if len(self.factory.workers) > 1:
                        if self.factory.lastCounter == 0:
                            print(
                                "Final results packet has been transmitted to the achilles_controller."
                            )

                        else:
                            self.factory.lastCounter = self.factory.lastCounter - 1
                    elif len(self.factory.workers) == 1:
                        print(
                            "Final results packet has been transmitted to the achilles_controller."
                        )

        elif "GET_CLUSTER_STATUS" in data:
            packet = {"CLUSTER_STATUS": True}
            for client in self.factory.clients:
                packet[str(client.CLIENT_ID)] = {}
                packet[str(client.CLIENT_ID)]["CLIENT_ID"] = str(client.CLIENT_ID)
                packet[str(client.CLIENT_ID)]["HOST"] = str(client.HOST)
                packet[str(client.CLIENT_ID)]["PORT"] = str(client.PORT)
                packet[str(client.CLIENT_ID)]["CPU_COUNT"] = str(client.CPU_COUNT)
                packet[str(client.CLIENT_ID)]["IP"] = str(client.IP)
                packet[str(client.CLIENT_ID)]["DATETIME_CONNECTED"] = str(
                    client.DATETIME_CONNECTED
                )
                packet[str(client.CLIENT_ID)]["AUTHENTICATED"] = str(
                    client.AUTHENTICATED
                )

            self.factory.achilles_controller.sendLine(cloudpickle.dumps(packet))

        elif "KILL_CLUSTER" in data:
            for client in self.factory.clients:
                client.sendLine(cloudpickle.dumps({"KILL_NODE": "KILL_NODE"}))

            for client in self.factory.clients:
                client.transport.loseConnection()

        elif "KILLED_CLUSTER" in data:
            reactor.stop()

        else:
            print(data)

    def startJob(self, func, args=(), args_path="", args_count=0):
        # Here is where the magic happens. Hungry consumers - feed them once and they keep
        # asking for more until the args are exhausted.

        # Flush settings in case another job has already been completed in this lifecycle.
        self.factory.args_path = ""
        self.factory.results = []
        self.factory.args_counter = 0

        print("MAP FUNC:", func)
        print("MAP ARGS:", args)
        ip_list = []
        ip_map = []
        workers_list = []
        cpu_total = 0
        try:
            self.factory.args = args(args_path)
        except TypeError:
            self.factory.args = iter(args)
        for client in self.factory.clients:
            if client.IP not in ip_list:
                ip_list.append(client.IP)
                ip_map.append((client.IP, client.CPU_COUNT))
                # Int may be unnecessary - check data type.
                cpu_total += int(client.CPU_COUNT)
            if client.AUTHENTICATED is False:
                workers_list.append(client)
        print("IP LIST:", ip_list)
        print("CPU TOTAL", cpu_total)
        print("CLIENTS CONNECTED:", self.factory.numProtocols)
        self.factory.ipMap = ip_map
        self.factory.workers = workers_list
        self.factory.lastCounter = len(self.factory.workers) - 1
        for client in self.factory.workers:
            client.sendLine(cloudpickle.dumps({"START_JOB": True, "FUNC": func}))
        self.factory.achilles_controller.sendLine(cloudpickle.dumps({"PROCEED": True}))

    def proceedWithJob(self):
        for worker in self.factory.workers:
            try:
                if isinstance(self.factory.args, GeneratorType):
                    args_counter, arg = next(self.factory.args)
                    packet = cloudpickle.dumps(
                        {"ARG": arg, "ARGS_COUNTER": args_counter}
                    )
                    worker.sendLine(packet)
                    print(f"Packet with arg {args_counter} sent to {worker.CLIENT_ID}")
                else:
                    packet = cloudpickle.dumps(
                        {
                            "ARG": next(self.factory.args),
                            "ARGS_COUNTER": self.factory.args_counter,
                        }
                    )
                    worker.sendLine(packet)
                    print(
                        f"Packet with arg {self.factory.args_counter} sent to {worker.CLIENT_ID}"
                    )
                    self.factory.args_counter = self.factory.args_counter + 1
            except StopIteration:
                print("The arguments iterable was empty.")


class AchillesServerFactory(Factory):

    protocol = AchillesServer
    numProtocols = 0
    totalProtocols = 0
    clients = []
    workers = []
    achilles_controller = None
    args = None
    args_path = ""
    args_count = 0
    args_counter = 0
    results = []
    lastCounter = 0
    ipMap = []
    response_mode = None
    HOST = ""
    PORT = 0
    USERNAME = ""
    SECRET_KEY = ""

    def __init__(self, host, port, username, secret_key):
        AchillesServerFactory.HOST = host
        AchillesServerFactory.PORT = port
        AchillesServerFactory.USERNAME = username
        AchillesServerFactory.SECRET_KEY = secret_key

    def buildProtocol(self, addr):
        return AchillesServer(factory=AchillesServerFactory)

    def gatherResults(self):
        final_results = []
        self.results = sorted(self.results, key=lambda k: k["ARGS_COUNTER"])
        for result in self.results:
            final_results.append(result["RESULT"])
        return final_results


def runAchillesServer():
    try:
        if __name__ != "__main__":
            import achilles

            dotenv_path = dirname(achilles.__file__) + "\\lineReceiver\\.env"
        else:
            basedir = abspath(dirname(__file__))
            dotenv_path = join(basedir, ".env")
        load_dotenv(dotenv_path, override=True)
        port = int(getenv("PORT"))
        host = getenv("HOST")
        username = getenv("USERNAME")
        secret_key = getenv("SECRET_KEY")

    except BaseException as e:
        print(
            f"No .env configuration file found ({e}). Follow the prompts below to generate one:"
        )
        host, port, username, secret_key = genConfig()

    endpoint = TCP4ServerEndpoint(reactor, port)
    endpoint.listen(AchillesServerFactory(host, port, username, secret_key))
    print(f"ALERT: achilles_server initiated on HOST {host} at PORT {port}")
    reactor.run()


def genConfig():
    if __name__ != "__main__":
        import achilles

        dotenv_path = dirname(achilles.__file__) + "\\lineReceiver\\"
    else:
        basedir = abspath(dirname(__file__))
        dotenv_path = join(basedir, ".env")
    host = input("Enter HOST IP address:\t")
    port = int(input("Enter host PORT to listen on:\t"))
    username = input("Enter USERNAME to require for authentication:\t")
    secret_key = getpass.getpass("Enter SECRET_KEY to require for authentication:\t")
    with open(dotenv_path + ".env", "w") as config_file:
        config_file.writelines(f"HOST={host}\n")
        config_file.writelines(f"PORT={port}\n")
        config_file.writelines(f"USERNAME='{username}'\n")
        config_file.writelines(f"SECRET_KEY='{secret_key}'\n")
        config_file.close()
        print(
            f"Successfully generated .env configuration file at {dotenv_path}.env. Use achilles_server.genConfig() to overwrite."
        )
    return host, port, username, secret_key


if __name__ == "__main__":
    runAchillesServer()



#!/usr/bin/env python3

import socket
import cloudpickle
from os import getenv

from multiprocessing import Pool
from dotenv import load_dotenv

from twisted.internet.protocol import Protocol
from twisted.internet.endpoints import TCP4ClientEndpoint, connectProtocol
from twisted.internet import reactor
import multiprocessing
from datetime import datetime


class CortexNode(Protocol):
    def __init__(self):
        load_dotenv()

        self.HOST = getenv("HOST")  # The server's hostname or IP address
        self.PORT = int(getenv("PORT"))  # The port used by the server
        self.USERNAME = getenv("USERNAME")
        self.SECRET_KEY = getenv("SECRET_KEY")
        self.connected = False
        self.client_id = -1
        self.func = None
        self.args = []

    def dataReceived(self, data):
        self.handleData(data)

    def handleData(self, data):
        data = cloudpickle.loads(data)
        if "GREETING" in data:
            greeting = data["GREETING"]
            client_id = data["CLIENT_ID"]
            self.client_id = client_id
            print("GREETING:", greeting)
            print("CLIENT_ID", client_id)
            packet = cloudpickle.dumps(
                {
                    "IP": socket.gethostbyname(socket.gethostname()),
                    "CPU_COUNT": multiprocessing.cpu_count(),
                    "DATETIME_CONNECTED": datetime.now(),
                    "CLIENT_ID": self.client_id,
                }
            )
            self.transport.write(packet)
        elif "START_JOB" in data:
            func = data["FUNC"]
            self.func = func
            packet = cloudpickle.dumps({"CLIENT_ID": self.client_id, "READY": True})
            print("START_JOB RESPONSE:", packet)
            self.transport.write(packet)
        elif "ARG" in data:
            print("ARG:", data["ARG"])
            with Pool(multiprocessing.cpu_count()) as p:
                result = p.map(self.func, data["ARG"])
                p.close()
            packet = cloudpickle.dumps(
                {"ARGS_COUNTER": data["ARGS_COUNTER"], "RESULT": result}
            )
            print("RESPONSE PACKET:", packet)
            self.transport.write(packet)
        elif "KILL_NODE" in data:
            self.transport.write(
                cloudpickle.dumps({"KILLED_CLUSTER": "KILLED_CLUSTER"})
            )
            self.transport.loseConnection()
            reactor.stop()


def runCortexNode():
    load_dotenv()
    endpoint = TCP4ClientEndpoint(reactor, getenv("HOST"), int(getenv("PORT")))
    d = connectProtocol(endpoint, CortexNode())

    reactor.run()


if __name__ == "__main__":
    runCortexNode()



#!/usr/bin/env python3

import socket
import cloudpickle
from sys import stderr
import sqlite3
import json

from dotenv import load_dotenv
import yaml
from os import getenv
from os.path import abspath, dirname, join
import getpass

from twisted.protocols.basic import LineReceiver
from twisted.internet.endpoints import TCP4ClientEndpoint, connectProtocol
from twisted.internet import reactor
import multiprocessing
from datetime import datetime


class AchillesController(LineReceiver):
    MAX_LENGTH = 999999

    def __init__(self, host, port, username, secret_key):

        self.HOST = host  # The server's hostname or IP address
        self.PORT = port  # The port used by the server
        self.USERNAME = username
        self.SECRET_KEY = secret_key
        self.response_mode = None
        self.sqlite_db_created = False
        self.sqlite_db = ""
        self.args_count = 0
        self.abs_counter = 0

    def lineReceived(self, data):
        data = cloudpickle.loads(data)
        if "GREETING" in data:
            greeting = data["GREETING"]
            print("GREETING:", greeting)
            packet = cloudpickle.dumps(
                {
                    "IP": socket.gethostbyname(socket.gethostname()),
                    "CPU_COUNT": multiprocessing.cpu_count(),
                    "DATETIME_CONNECTED": datetime.now(),
                    "USERNAME": self.USERNAME,
                    "SECRET_KEY": self.SECRET_KEY,
                }
            )
            self.sendLine(packet)

        elif "AUTHENTICATED" in data:
            if data["AUTHENTICATED"] is True:
                print("ALERT: Authentication successful!\n")
                self.command_interface()
            else:
                stderr.write("WARNING: Authentication failed.")
                self.transport.loseConnection()

        elif "PROCEED" in data:
            proceed = input("Press ENTER if ready to proceed:\t")
            if proceed == "":
                print(
                    "PROCEEDING WITH DISTRIBUTING ARGUMENTS AMONGST THE CONNECTED NODES..."
                )
                self.sendLine(cloudpickle.dumps({"VERIFY": True}))
            else:
                stderr.write("ALERT: Job cancelled.")
                self.command_interface()

        elif "RESULT" in data and self.response_mode == "STREAM":
            # If STREAM is your chosen response mode, handle results packets here.
            print(data)
            # Prematurely calls command_interface in certain instances when the last results packet is returned while jobs remain outstanding.
            # if data["ARGS_COUNTER"] == self.args_count - 1:
            # self.command_interface()

        elif "RESULT" in data and self.response_mode == "SQLITE":
            if self.sqlite_db_created is False:
                self.sqlite_db = datetime.now().strftime("%Y%m%d %H%M%S") + ".db"
                conn = sqlite3.connect(self.sqlite_db)
                c = conn.cursor()
                c.execute(
                    "CREATE TABLE results (args_counter real, abs_counter real, result text)"
                )
                self.sqlite_db_created = True
                if len(data["RESULT"]) == 1:
                    try:
                        instruction = f"""INSERT INTO results VALUES ({data['ARGS_COUNTER']}, {self.abs_counter}, {data['RESULT'][0]})"""
                        # print(instruction)
                        c.execute(instruction)
                        self.abs_counter = self.abs_counter + 1
                    except BaseException:
                        instruction = f"""INSERT INTO results VALUES ({data['ARGS_COUNTER']}, {self.abs_counter}, {json.dumps(data['RESULT'][0])})"""
                        # print(instruction)
                        c.execute(instruction)
                        self.abs_counter = self.abs_counter + 1
                    # finally:
                    # if data["ARGS_COUNTER"] == self.args_count - 1:
                    # self.command_interface()
                else:
                    for i in range(len(data["RESULT"])):
                        try:
                            instruction = f"""INSERT INTO results VALUES ({data['ARGS_COUNTER']}, {self.abs_counter}, {data['RESULT'][i]})"""
                            # print(instruction)
                            c.execute(instruction)
                            self.abs_counter = self.abs_counter + 1
                        except BaseException:
                            instruction = f"""INSERT INTO results VALUES ({data['ARGS_COUNTER']}, {self.abs_counter}, {json.dumps(data['RESULT'][i])})"""
                            # print(instruction)
                            c.execute(instruction)
                            self.abs_counter + self.abs_counter + 1

                    # if data["ARGS_COUNTER"] == self.args_count - 1:
                    # self.command_interface()
                c.close()

            else:
                conn = sqlite3.connect(self.sqlite_db)
                c = conn.cursor()
                if len(data["RESULT"]) == 1:
                    try:
                        instruction = f"""INSERT INTO results VALUES ({data['ARGS_COUNTER']}, {self.abs_counter}, {data['RESULT'][0]})"""
                        # print(instruction)
                        c.execute(instruction)
                        self.abs_counter = self.abs_counter + 1
                    except BaseException:
                        instruction = f"""INSERT INTO results VALUES ({data['ARGS_COUNTER']}, {self.abs_counter}, {json.dumps(data['RESULT'][0])})"""
                        # print(instruction)
                        c.execute(instruction)
                        self.abs_counter = self.abs_counter + 1
                    # finally:
                    # if data["ARGS_COUNTER"] == self.args_count - 1:
                    # self.command_interface()
                else:
                    for i in range(len(data["RESULT"])):
                        try:
                            instruction = f"""INSERT INTO results VALUES ({data['ARGS_COUNTER']}, {self.abs_counter}, {data['RESULT'][i]})"""
                            # print(instruction)
                            c.execute(instruction)
                            self.abs_counter = self.abs_counter + 1
                        except BaseException:
                            instruction = f"""INSERT INTO results VALUES ({data['ARGS_COUNTER']}, {self.abs_counter}, {json.dumps(data['RESULT'][i])})"""
                            # print(instruction)
                            c.execute(instruction)
                            self.abs_counter + self.abs_counter + 1
                    # if data["ARGS_COUNTER"] == self.args_count - 1:
                    # self.command_interface()
                c.close()

        elif "FINAL_RESULT" in data:
            print("FINAL RESULT:", data["FINAL_RESULT"])
            self.command_interface()

        elif "CLUSTER_STATUS" in data:
            print("CLUSTER STATUS:", data)
            self.command_interface()

        elif "KILL_NODE" in data:
            stderr.write(
                "ALERT: All achilles_nodes have been disconnected from the cluster. The achilles_server is running and accepting connections."
            )
            reactor.stop()

        else:
            print(data)
            self.command_interface()

    def achilles_compute(
        self,
        achilles_config_path="",
        args=(),
        callback=None,
        callback_args=(),
        group="default",
        response_mode="",
    ):
        from achilles_function import achilles_function, achilles_args

        achilles_config_path = achilles_config_path
        with open(achilles_config_path, "r") as f:
            achilles_config = yaml.load(f, Loader=yaml.Loader)
        try:
            args_path = achilles_config["ARGS_PATH"]
        except KeyError:
            args_path = None
        try:
            achilles_args = achilles_config["ARGS"]
        except KeyError:
            achilles_args = achilles_args
        self.args_count = 0
        try:
            for arg in achilles_args(args_path):
                self.args_count = self.args_count + 1
        except TypeError:
            for arg in achilles_args:
                self.args_count = self.args_count + 1
        print("ARGS COUNT:", self.args_count)
        self.response_mode = response_mode
        modules = achilles_config["MODULES"]
        callback = achilles_args["CALLBACK"]
        callback_args = achilles_config["CALLBACK_ARGS"]
        group = achilles_config["GROUP"]

        packet = cloudpickle.dumps(
            {
                "FUNC": achilles_function,
                "ARGS": achilles_args,
                "ARGS_PATH": args_path,
                "ARGS_COUNT": self.args_count,
                "MODULES": modules,
                "CALLBACK": callback,
                "CALLBACK_ARGS": callback_args,
                "GROUP": group,
                "RESPONSE_MODE": response_mode,
            }
        )
        self.sendLine(packet)

    def get_cluster_status(self):
        packet = cloudpickle.dumps({"GET_CLUSTER_STATUS": "GET_CLUSTER_STATUS"})
        self.sendLine(packet)
        print("ALERT: Requested cluster status.")

    def kill_cluster(self):
        confirm_kill_cluster = input(
            "WARNING: Are you absolutely sure that you want to kill the cluster? Enter YES to proceed."
        )
        if confirm_kill_cluster == "YES":
            packet = cloudpickle.dumps({"KILL_CLUSTER": "KILL_CLUSTER"})
            self.sendLine(packet)
        else:
            stderr.write("ALERT: kill_cluster aborted.")
            self.command_interface()

    def command_interface(self):
        command = input("Achilles cluster is ready to accept commands:\t")
        if command == "achilles_compute":
            self.init_achilles_compute()
        elif command == "cluster_status":
            self.get_cluster_status()
        elif command == "kill_cluster":
            self.kill_cluster()
        elif command == "help":
            print("\n-------\nCommands:")
            print("achilles_compute, cluster_status, kill_cluster, help\n-------\n")
            self.command_interface()
        else:
            stderr.write(
                "Sorry, that command is not recognized. Type 'help' for a list of commands.\n"
            )
            self.command_interface()

    def init_achilles_compute(self):
        achilles_config_path = input(
            "Enter path to achilles_config.yaml to begin job:\t"
        )
        response_mode = input(
            f"Enter desired response mode (OBJECT, SQLITE, or STREAM):\t"
        )
        if response_mode in ["OBJECT", "SQLITE", "STREAM"]:
            self.achilles_compute(
                achilles_config_path=achilles_config_path, response_mode=response_mode
            )
        else:
            stderr.write(
                "Sorry, that response mode is not recognized. Please choose OBJECT, SQLITE or STREAM.\n"
            )
            self.init_achilles_compute()


def runAchillesController():
    try:
        if __name__ != "__main__":
            import achilles

            dotenv_path = dirname(achilles.__file__) + "\\lineReceiver\\.env"
        else:
            basedir = abspath(dirname(__file__))
            dotenv_path = join(basedir, ".env")
        load_dotenv(dotenv_path, override=True)
        port = int(getenv("PORT"))
        host = getenv("HOST")
        username = getenv("USERNAME")
        secret_key = getenv("SECRET_KEY")

    except BaseException as e:
        print(
            f"No .env configuration file found ({e}). Follow the prompts below to generate one:"
        )
        host, port, username, secret_key = genConfig()

    endpoint = TCP4ClientEndpoint(reactor, host, port)
    d = connectProtocol(endpoint, AchillesController(host, port, username, secret_key))

    reactor.run()


def genConfig():
    if __name__ != "__main__":
        import achilles

        dotenv_path = dirname(achilles.__file__) + "\\lineReceiver\\"
    else:
        basedir = abspath(dirname(__file__))
        dotenv_path = join(basedir, ".env")
    host = input("Enter HOST IP address:\t")
    port = int(input("Enter HOST port to listen on:\t"))
    username = input("Enter USERNAME to require for authentication:\t")
    secret_key = getpass.getpass("Enter SECRET_KEY to require for authentication:\t")
    with open(dotenv_path + ".env", "w") as config_file:
        config_file.writelines(f"HOST={host}\n")
        config_file.writelines(f"PORT={port}\n")
        config_file.writelines(f"USERNAME='{username}'\n")
        config_file.writelines(f"SECRET_KEY='{secret_key}'\n")
        config_file.close()
        print(
            f"Successfully generated .env configuration file at {dotenv_path}.env. Use achilles_controller.genConfig() to overwrite."
        )
    return host, port, username, secret_key


if __name__ == "__main__":
    runAchillesController()



#!/usr/bin/env python3

import ast

from requests import get
from html2text import html2text


def achilles_args(args_path):
    args_counter = -1
    with open(args_path, "r") as args:
        for arg in args:
            args_counter = args_counter + 1
            yield args_counter, ast.literal_eval(arg)


def achilles_function(arg):
    return arg ** 2


def achilles_function2(arg):
    r = get(
        arg,
        headers={"User-Agent": "Mozilla/5.0", "referrer": "https://l.facebook.com/"},
    )
    raw_html = r.text
    markdown_text = html2text(raw_html)

    return markdown_text
